# Reference
<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

**Classes**

_Public Classes_

* [`nservicebusservicecontrol`](#nservicebusservicecontrol): Installs and configures Particular's Service Control Monitoring Tool.

_Private Classes_

* `nservicebusservicecontrol::config`: This class handles the configuration of servicecontrol.
* `nservicebusservicecontrol::install`: This class handles the management of the servicecontrol installer and package.

**Defined types**

* [`nservicebusservicecontrol::audit_instance`](#nservicebusservicecontrolaudit_instance): Manages Service Control Instances.
* [`nservicebusservicecontrol::instance`](#nservicebusservicecontrolinstance): Manages Service Control Instances.
* [`nservicebusservicecontrol::monitoring_instance`](#nservicebusservicecontrolmonitoring_instance): Manages Service Control Monitoring Instances.

**Data types**

* [`Nservicebusservicecontrol::Log_level`](#nservicebusservicecontrollog_level): 
* [`Nservicebusservicecontrol::Transport`](#nservicebusservicecontroltransport): https://github.com/Particular/ServiceControl/blob/develop/src/ServiceControlInstaller.Engine/Instances/TransportNames.cs

**Tasks**

* [`get_instances`](#get_instances): Gets all servicecontrol instances.

**Plans**

* [`nservicebusservicecontrol::import_failed_messages`](#nservicebusservicecontrolimport_failed_messages): Imports failed error or audit messages.

## Classes

### nservicebusservicecontrol

Installs and configures Particular's Service Control Monitoring Tool.

#### Parameters

The following parameters are available in the `nservicebusservicecontrol` class.

##### `package_ensure`

Data type: `String`

Whether to install the ServiceControl package.

Default value: 'present'

##### `package_source`

Data type: `Optional[String]`

The package source for the package.

Default value: `undef`

##### `license_xml`

Data type: `Optional[String]`

A valid NServiceBus XML License.

Default value: ''

## Defined types

### nservicebusservicecontrol::audit_instance

Manages Service Control Instances.

#### Parameters

The following parameters are available in the `nservicebusservicecontrol::audit_instance` defined type.

##### `ensure`

Data type: `Enum['present', 'absent']`

Specifies whether the instance should exist.

##### `service_control_queue_address`

Data type: `String`

The ServiceControl queue name to use for plugin messages (e.g. Heartbeats, Custom Checks, Saga Audit, etc ).

##### `instance_name`

Data type: `String`

Specify the name of the ServiceControl Instance (title).

Default value: $title

##### `install_path`

Data type: `Stdlib::Absolutepath`

Specify the directory to use for this ServiceControl Instance.

Default value: "C:\\Program Files (x86)\\Particular Software\\${instance_name}"

##### `log_path`

Data type: `Stdlib::Absolutepath`

Specify the directory to use for this ServiceControl Logs.

Default value: "C:\\ProgramData\\Particular\\ServiceControl\\${instance_name}\\Logs"

##### `db_path`

Data type: `Stdlib::Absolutepath`

Specify the directory that will contain the nservicebusservicecontrol database for this ServiceControl Instance.

Default value: "C:\\ProgramData\\Particular\\ServiceControl\\${instance_name}\\DB"

##### `db_index_storage_path`

Data type: `Stdlib::Absolutepath`

Specify the path for the indexes on disk.

Default value: "${db_path}\\Indexes"

##### `db_logs_path`

Data type: `Stdlib::Absolutepath`

Specify the path for the Esent logs on disk.

Default value: "${db_path}\\logs"

##### `instance_log_level`

Data type: `Nservicebusservicecontrol::Log_level`

Specify the level of logging that should be used in ServiceControl logs.

Default value: 'Warn'

##### `host_name`

Data type: `Stdlib::Fqdn`

Specify the hostname to use in the URLACL.

Default value: 'localhost'

##### `port`

Data type: `Stdlib::Port`

Specify the port number to listen on. If this is the only ServiceControl instance then 44444 is recommended.

Default value: 44444

##### `database_maintenance_port`

Data type: `Stdlib::Port`

Specify the database maintenance port number to listen on. If this is the only ServiceControl instance then 44445 is recommended.

Default value: 44445

##### `expose_ravendb`

Data type: `Boolean`

Specify if the embedded ravendb database should be accessible outside of maintenance mode.

Default value: `false`

##### `ravendb_log_level`

Data type: `Nservicebusservicecontrol::Log_level`

Specify the level of logging that should be used in ravendb logs.

Default value: 'Warn'

##### `transport`

Data type: `Nservicebusservicecontrol::Transport`

Specify the NServiceBus Transport to use.

Default value: 'MSMQ'

##### `display_name`

Data type: `String`

Specify the Windows Service Display name. If unspecified the instance name will be used.

Default value: $instance_name

##### `connection_string`

Data type: `Optional[String]`

Specify the connection string to use to connect to the queuing system.

Default value: `undef`

##### `description`

Data type: `String`

Specify the description to use on the Windows Service for this instance.

Default value: 'A ServiceControl Audit Instance'

##### `service_account`

Data type: `String`

The Account to run the Windows service.

Default value: 'LocalSystem'

##### `service_account_password`

Data type: `Optional[String]`

The password for the ServiceAccount.

Default value: `undef`

##### `service_restart_on_config_change`

Data type: `Boolean`

Specify if the servicecontrol instance's windows service should be restarted to pick up changes to its configuration file.

Default value: `true`

##### `audit_retention_period`

Data type: `String`

Specify thd grace period that faulted messages are kept before they are deleted.

Default value: '30.00:00:00'

##### `time_to_restart_audit_ingestion_after_failure`

Data type: `String`

Specify the maximum time delay to wait before restarting the audit ingestion pipeline after detecting a connection problem. This setting was introduced in ServiceControl version 4.4.1.

Default value: '00.00:01:00'

##### `expiration_process_timer_in_seconds`

Data type: `Integer`

Specifies the number of seconds to wait between checking for expired messages.

Default value: 600

##### `expiration_process_batch_size`

Data type: `Integer`

Specifies the batch size to use when checking for expired messages.

Default value: 65512

##### `data_space_remaining_threshold`

Data type: `Integer`

The percentage threshold for the Message database storage space check. If the remaining hard drive space drops below this threshold (as a percentage of the total space on the drive), then the check will fail, alerting the user.

Default value: 20

##### `max_body_size_to_store`

Data type: `Integer`

Specifies the upper limit on body size to be configured.

Default value: 102400

##### `http_default_connection_limit`

Data type: `Integer`

Specifies the maximum number of concurrent connections allowed by ServiceControl.

Default value: 100

##### `service_manage`

Data type: `Boolean`

Specifies whether or not to manage the desired state of the windows service for this instance.

Default value: `true`

##### `skip_queue_creation`

Data type: `Boolean`

Normally an instance will attempt to create the queues that it uses. If this flag is set, queue creation will be skipped.

Default value: `false`

##### `remove_db_on_delete`

Data type: `Boolean`

Remove service control instance ravendb database when instance is deleted.

Default value: `false`

##### `remove_logs_on_delete`

Data type: `Boolean`

Remove service control instance logs when instance is deleted.

Default value: `false`

##### `automatic_instance_upgrades`

Data type: `Boolean`

Automatically upgrade the service control monitoring instance when a new version of servicecontrol is installed.

Default value: `true`

##### `audit_queue`

Data type: `String`



Default value: 'audit'

##### `audit_log_queue`

Data type: `Optional[String]`



Default value: 'audit.log'

##### `forward_audit_messages`

Data type: `Boolean`



Default value: `false`

### nservicebusservicecontrol::instance

Manages Service Control Instances.

#### Parameters

The following parameters are available in the `nservicebusservicecontrol::instance` defined type.

##### `ensure`

Data type: `Enum['present', 'absent']`

Specifies whether the instance should exist.

##### `instance_name`

Data type: `String`

Specify the name of the ServiceControl Instance (title).

Default value: $title

##### `install_path`

Data type: `Stdlib::Absolutepath`

Specify the directory to use for this ServiceControl Instance.

Default value: "C:\\Program Files (x86)\\Particular Software\\${instance_name}"

##### `log_path`

Data type: `Stdlib::Absolutepath`

Specify the directory to use for this ServiceControl Logs.

Default value: "C:\\ProgramData\\Particular\\ServiceControl\\${instance_name}\\Logs"

##### `db_path`

Data type: `Stdlib::Absolutepath`

Specify the directory that will contain the nservicebusservicecontrol database for this ServiceControl Instance.

Default value: "C:\\ProgramData\\Particular\\ServiceControl\\${instance_name}\\DB"

##### `db_index_storage_path`

Data type: `Stdlib::Absolutepath`

Specify the path for the indexes on disk.

Default value: "${db_path}\\Indexes"

##### `db_logs_path`

Data type: `Stdlib::Absolutepath`

Specify the path for the Esent logs on disk.

Default value: "${db_path}\\logs"

##### `instance_log_level`

Data type: `Nservicebusservicecontrol::Log_level`

Specify the level of logging that should be used in ServiceControl logs.

Default value: 'Warn'

##### `host_name`

Data type: `Stdlib::Fqdn`

Specify the hostname to use in the URLACL.

Default value: 'localhost'

##### `port`

Data type: `Stdlib::Port`

Specify the port number to listen on. If this is the only ServiceControl instance then 33333 is recommended.

Default value: 33333

##### `database_maintenance_port`

Data type: `Stdlib::Port`

Specify the database maintenance port number to listen on. If this is the only ServiceControl instance then 33334 is recommended.

Default value: 33334

##### `remote_instances`

Data type: `Optional[Array[String]]`

Specify an optional array of remote instances.

Default value: []

##### `expose_ravendb`

Data type: `Boolean`

Specify if the embedded ravendb database should be accessible outside of maintenance mode.

Default value: `false`

##### `ravendb_log_level`

Data type: `Nservicebusservicecontrol::Log_level`

Specify the level of logging that should be used in ravendb logs.

Default value: 'Warn'

##### `error_queue`

Data type: `String`

Specify ErrorQueue name to consume messages from.

Default value: 'error'

##### `error_log_queue`

Data type: `Optional[String]`

Specify Queue name to forward error messages to.

Default value: 'error.log'

##### `transport`

Data type: `Nservicebusservicecontrol::Transport`

Specify the NServiceBus Transport to use.

Default value: 'MSMQ'

##### `display_name`

Data type: `String`

Specify the Windows Service Display name. If unspecified the instance name will be used.

Default value: $instance_name

##### `connection_string`

Data type: `Optional[String]`

Specify the connection string to use to connect to the queuing system.

Default value: `undef`

##### `description`

Data type: `String`

Specify the description to use on the Windows Service for this instance.

Default value: 'A ServiceControl Instance'

##### `forward_error_messages`

Data type: `Boolean`

Specify if audit messages are forwarded to the queue specified by ErrorLogQueue.

Default value: `false`

##### `service_account`

Data type: `String`

The Account to run the Windows service.

Default value: 'LocalSystem'

##### `service_account_password`

Data type: `Optional[String]`

The password for the ServiceAccount.

Default value: `undef`

##### `service_restart_on_config_change`

Data type: `Boolean`

Specify if the service control instance's windows service should be restarted to pick up changes to its configuration file.

Default value: `true`

##### `error_retention_period`

Data type: `String`

Specify thd grace period that faulted messages are kept before they are deleted.

Default value: '15.00:00:00'

##### `time_to_restart_error_ingestion_after_failure`

Data type: `String`

Specify the maximum time delay to wait before restarting the error ingestion pipeline after detecting a connection problem. This setting was introduced in ServiceControl version 4.4.1.

Default value: '00.00:01:00'

##### `event_retention_period`

Data type: `String`

Specifies the period to keep event logs before they are deleted.

Default value: '14.00:00:00'

##### `expiration_process_timer_in_seconds`

Data type: `Integer`

Specifies the number of seconds to wait between checking for expired messages.

Default value: 600

##### `expiration_process_batch_size`

Data type: `Integer`

Specifies the batch size to use when checking for expired messages.

Default value: 65512

##### `data_space_remaining_threshold`

Data type: `Integer`

The percentage threshold for the Message database storage space check. If the remaining hard drive space drops below this threshold (as a percentage of the total space on the drive), then the check will fail, alerting the user.

Default value: 20

##### `http_default_connection_limit`

Data type: `Integer`

Specifies the maximum number of concurrent connections allowed by ServiceControl.

Default value: 100

##### `heartbeat_grace_period`

Data type: `String`

Specifies the period that defines whether an endpoint is considered alive or not since the last received heartbeat.

Default value: '00:00:40'

##### `allow_message_editing`

Data type: `Boolean`

Enables the ability for servicepulse to allow users to edit failed messages before being retried.

Default value: `false`

##### `service_manage`

Data type: `Boolean`

Specifies whether or not to manage the desired state of the windows service for this instance.

Default value: `true`

##### `skip_queue_creation`

Data type: `Boolean`

Normally an instance will attempt to create the queues that it uses. If this flag is set, queue creation will be skipped.

Default value: `false`

##### `remove_db_on_delete`

Data type: `Boolean`

Remove service control instance ravendb database when instance is deleted.

Default value: `false`

##### `remove_logs_on_delete`

Data type: `Boolean`

Remove service control instance logs when instance is deleted.

Default value: `false`

##### `automatic_instance_upgrades`

Data type: `Boolean`

Automatically upgrade the service control monitoring instance when a new version of servicecontrol is installed.

Default value: `true`

### nservicebusservicecontrol::monitoring_instance

Manages Service Control Monitoring Instances.

#### Parameters

The following parameters are available in the `nservicebusservicecontrol::monitoring_instance` defined type.

##### `ensure`

Data type: `Enum['present', 'absent']`

Specifies whether the monitoring instance should exist.

##### `instance_name`

Data type: `String`

Specify the name of the ServiceControl Monitoring Instance (title).

Default value: $title

##### `install_path`

Data type: `Stdlib::Absolutepath`

Specify the directory to use for this ServiceControl Monitoring Instance.

Default value: "C:\\Program Files (x86)\\Particular Software\\${instance_name}"

##### `log_path`

Data type: `Stdlib::Absolutepath`

Specify the directory to use for this ServiceControl Monitoring Logs.

Default value: "C:\\ProgramData\\Particular\\ServiceControl\\${instance_name}\\Logs"

##### `instance_log_level`

Data type: `Nservicebusservicecontrol::Log_level`

Specify the level of logging that should be used in ServiceControl Monitoring logs.

Default value: 'Warn'

##### `host_name`

Data type: `Stdlib::Fqdn`

Specify the hostname to use in the URLACL.

Default value: 'localhost'

##### `port`

Data type: `Stdlib::Port`

Specify the port number to listen on. If this is the only ServiceControl Monitoring instance then 33633 is recommended.

Default value: 33633

##### `error_queue`

Data type: `String`

Specify the ErrorQueue name.

Default value: 'error'

##### `transport`

Data type: `Nservicebusservicecontrol::Transport`

Specify the NServiceBus Transport to use.

Default value: 'MSMQ'

##### `display_name`

Data type: `String`

Specify the Windows Service Display name. If unspecified the monitoring instance name will be used.

Default value: $instance_name

##### `connection_string`

Data type: `Optional[String]`

Specify the connection string to use to connect to the queuing system.

Default value: `undef`

##### `description`

Data type: `String`

Specify the description to use on the Windows Service for this instance.

Default value: 'A Monitoring Instance'

##### `service_account`

Data type: `String`

The Account to run the Windows service.

Default value: 'LocalSystem'

##### `service_account_password`

Data type: `Optional[String]`

The password for the ServiceAccount.

Default value: `undef`

##### `service_restart_on_config_change`

Data type: `Boolean`

Specify if the servicecontrol monitoring instance's windows service should be restarted to pick up changes to its configuration file.

Default value: `true`

##### `service_manage`

Data type: `Boolean`

Specifies whether or not to manage the desired state of the windows service for this instance.

Default value: `true`

##### `skip_queue_creation`

Data type: `Boolean`

Normally an instance will attempt to create the queues that it uses. If this flag is set, queue creation will be skipped.

Default value: `false`

##### `remove_logs_on_delete`

Data type: `Boolean`

Specifies if the service control logs should be deleted with the instance.

Default value: `false`

##### `automatic_instance_upgrades`

Data type: `Boolean`

Automatically upgrade the service control monitoring instance when a new version of servicecontrol is installed.

Default value: `true`

## Data types

### Nservicebusservicecontrol::Log_level

The Nservicebusservicecontrol::Log_level data type.

Alias of `Enum['Trace', 'Debug', 'Info', 'Warn', 'Error', 'Fatal', 'Off']`

### Nservicebusservicecontrol::Transport

https://github.com/Particular/ServiceControl/blob/develop/src/ServiceControlInstaller.Engine/Instances/TransportNames.cs

Alias of `Enum['RabbitMQ - Conventional routing topology', 'SQL Server', 'MSMQ', 'Azure Storage Queue', 'Azure Service Bus', 'AmazonSQS']`

## Tasks

### get_instances

Gets all servicecontrol instances.

**Supports noop?** false

## Plans

### nservicebusservicecontrol::import_failed_messages

Imports failed error or audit messages.

#### Parameters

The following parameters are available in the `nservicebusservicecontrol::import_failed_messages` plan.

##### `targets`

Data type: `TargetSpec`

Targets to import failed messags on.

##### `instance_name`

Data type: `String[1]`

The name of the servicecontrol instance.

##### `instance_type`

Data type: `Enum['error', 'audit']`

The servicecontrol instance type (Audit or Error).

